@@ -16,8 +16,7 @@
 # under the License.
 from __future__ import annotations
 
-from collections.abc import Iterator
-from typing import Callable
+from collections.abc import Callable, Iterator
 
 from kubernetes.utils import FailToCreateError
 
